//
//  APMServicesController.h
//  APMServices
//
//  Created by Vincent Ducastel on 24/07/2015.
//  Copyright (c) 2015 Moonmiles. All rights reserved.
//

#import <CoreLocation/CoreLocation.h>

@class APMEarn;
@class APMUser;
@class APMProg;
@class APMGenerosity;
@class APMGift;
@class APMDevice;
@class APMAction;
@class APMBurn;
@class APMVersions;

@protocol APMServicesListener <NSObject>

-(void)apmServicesOnLoading;
-(void)apmServicesReady:(BOOL)newInfos;
-(void)apmServicesDisable;
-(void)apmServicesUserWinPoints:(APMEarn*)earn user:(APMUser*)user;
-(void)apmServicesShowError:(NSError*)error;

@end

@interface APMServices : NSObject<CLLocationManagerDelegate>
{
    BOOL _applicationInBackground; //Use for catch event applicationWillResignActive and applicationDidBecomeActive : can laucnh initAPM
    BOOL _applicationHandleOpenUrl; //Use for catch event applicationOpenUrl : if true, initAPM don't launched

    NSString *_apmScheme;
    NSInteger _partnerLevel; //Partner level, directly setted by APMProg
    
    APMUser *_user;
    APMProg *_prog;
    APMGenerosity *_generosity;
    APMGift *_gift;
    APMDevice *_device;
    
    NSMutableDictionary *_registeredActions;
    NSString *_classID;
    NSMutableArray *_frequencies;
    NSMutableArray *_values;

    NSMutableArray *_stackTask;
    NSString *_tagIDFounded;
    
    //Geoloc
    CLLocationManager *_locationManager;
    NSInteger _tryUpdateLocation;
}

@property (weak, nonatomic) id<APMServicesListener> servicesListener;

@property(nonatomic, assign) BOOL sdkLifeCycleEnabled; //If true, life cycle is enabled, false life cycle is disabled
@property(nonatomic, assign) BOOL sdkIsLoading; //Block lifeCycle of badge if userClientConnect or userClientUpdate (APMServicesUser) is launched
@property(nonatomic, assign) BOOL geolocEnabled;

@property(nonatomic, strong) NSString *baseUrl;
@property(nonatomic, strong) NSString *apmVersionCode;
@property(nonatomic, strong) NSString *apmVersionName;
@property(nonatomic, strong) NSString *appID;
@property(nonatomic, strong) NSString *partnerID;
@property(nonatomic, strong) NSString *partnerSecret;
@property(nonatomic, strong) NSMutableArray *status;
@property(nonatomic, strong) NSMutableArray *generosities;
@property(nonatomic, strong) APMVersions* versions;

@property(nonatomic, strong) NSMutableDictionary* theme;

//Init
+(APMServices *)sharedInstance;
-(void)initAppWithPartnerID:(NSString*)partnerID withPartnerSecret:(NSString*)partnerSecret;
-(void)initAppWithPartnerID:(NSString*)partnerID withPartnerSecret:(NSString*)partnerSecret withAppId:(NSString*)appId;

//Cycle de vie
-(void)viewWillAppear:(BOOL)animated;
-(void)viewWillAppear:(BOOL)animated classID:(NSString*)classID;
-(void)viewDidAppear:(BOOL)animated;
-(void)viewWillDisappear:(BOOL)animated;
-(void)viewDidDisappear:(BOOL)animated;

//Services
-(void)registerAction:(APMAction*)action;
-(void)removeActionWithActionName:(NSString*)actionName;
-(NSDictionary*)getActionForClassId:(NSString*)classId;
-(void)selectClassID:(NSString*)classID;
-(void)triggerAction:(NSString*)actionName;
-(void)triggerAuto __attribute__((deprecated(("triggerAuto is DEPRECATED, use triggerAction"))));
-(void)triggerAuto:(NSString*)actionName __attribute__((deprecated(("triggerAuto is DEPRECATED, use triggerAction"))));
-(BOOL)handleOpenURL:(NSURL*)url;

//Getter and Setter
-(BOOL)isInitialLevel;
-(void)setUser:(APMUser*)user;
-(APMUser*)user;
-(APMProg*)prog;
-(APMGenerosity*)generosity;
-(APMGift*)gift;
-(APMDevice*)device;
-(NSMutableArray*)frequencies;
-(NSMutableArray*)values;
-(NSString*)classId;
-(BOOL)debugMode;
-(void)setDebugMode:(BOOL)debugMode;
-(BOOL)servicesIsOnError;
-(BOOL)servicesIsDisabled;

//Other
-(void)resetAll;

@end
